## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this program; If not, see <http://www.gnu.org/licenses/>.

function handle = auopen(path)
  if (nargin != 1)
    usage("handle = auopen('filename.ext')");
  end
  
  handle.rate = 8000;
  handle.sampleformat = 'ulaw';
  handle.sampleOffset=0; # offset of next read in samples
  handle.dataOffset=0; # byte position of start of samples
  handle.bits=0; # bits per sample
  handle.channels=0;
  
  ext = rindex(path, '.');
  if (ext == 0)
    usage('h = auopen(filename.ext)');
  end
  ext = tolower(substr(path, ext+1, length(path)-ext));

  [handle.file, msg] = fopen(path, 'rb');
  if (handle.file == -1)
    error([ msg, ": ", path]);
  end

  msg = sprintf('Invalid audio header: %s', path);
  ## Microsoft .wav format
  if strcmp(ext,'wav') 

    ## Header format obtained from sox/wav.c
    ## April 15, 1992
    ## Copyright 1992 Rick Richardson
    ## Copyright 1991 Lance Norskog And Sundry Contributors
    ## This source code is freely redistributable and may be used for
    ## any purpose.  This copyright notice must be maintained. 
    ## Lance Norskog And Sundry Contributors are not responsible for 
    ## the consequences of using this software.

    ## check the file magic header bytes
    handle.arch = 'ieee-le';
    str = char(fread(handle.file, 4, 'char')');
    if !strcmp(str, 'RIFF')
      error(msg);
    end
    len = fread(handle.file, 1, 'int32', 0, handle.arch);
    str = char(fread(handle.file, 4, 'char')');
    if !strcmp(str, 'WAVE')
      error(msg);
    end

    ## skip to the "fmt " section, ignoring everything else
    while (1)
      if feof(handle.file)
      	error(msg);
      end
      str = char(fread(handle.file, 4, 'char')');
      len = fread(handle.file, 1, 'int32', 0, handle.arch);
      if strcmp(str, 'fmt ')
	break;
      end
      fseek(handle.file, len, SEEK_CUR);
    end

    ## read the "fmt " section
    formatid = fread(handle.file, 1, 'int16', 0, handle.arch);
    handle.channels = fread(handle.file, 1, 'int16', 0, handle.arch);
    handle.rate = fread(handle.file, 1, 'int32', 0, handle.arch);
    fread(handle.file, 1, 'int32', 0, handle.arch);
    fread(handle.file, 1, 'int16', 0, handle.arch);
    handle.bits = fread(handle.file, 1, 'int16', 0, handle.arch);
    fseek(handle.file, len-16, SEEK_CUR);

    ## skip to the "data" section, ignoring everything else
    while (1)
      if feof(handle.file)
      	error(msg);
      end
      str = char(fread(handle.file, 4, 'char')');
      len = fread(handle.file, 1, 'int32', 0, handle.arch);
      if strcmp(str, 'data')
	break;
      end
      fseek(handle.file, len, SEEK_CUR);
    end

    if (formatid == 1)
      if handle.bits == 8
      	handle.sampleformat = 'uchar';
	handle.precision = 'uchar';
        handle.samples = len;
      elseif handle.bits == 16
      	handle.sampleformat = 'int16';
	handle.precision = 'int16';
        handle.samples = len/2;
      elseif handle.bits == 24
      	handle.sampleformat = 'int24';
	handle.precision = 'int24';
        handle.samples = len/3;
      elseif handle.bits == 32
	handle.sampleformat = 'int32';
	handle.precision = 'int32';
        handle.samples = len/4;
      else
       	error(msg);
      endif
    elseif (formatid == 3)
      if handle.bits == 32
	handle.sampleformat = 'float';
	handle.precision = 'float';
	handle.samples = len/4;
      elseif handle.bits == 64
	handle.sampleformat = 'double';
	handle.precision = 'double';
	handle.samples = len/8;
      else
	error(msg);
      endif
    elseif (formatid == 6 && handle.bits == 8)
      handle.sampleformat = 'alaw';
      handle.precision = 'uchar';
      handle.samples = len;
    elseif (formatid == 7 && handle.bits == 8)
      handle.sampleformat = 'ulaw';
      handle.precision = 'uchar';
      handle.samples = len;
    else
      error(msg);
      return;
    endif

  ## Sun .au format
  elseif strcmp(ext, 'au')

    ## Header format obtained from sox/au.c
    ## September 25, 1991
    ## Copyright 1991 Guido van Rossum And Sundry Contributors
    ## This source code is freely redistributable and may be used for
    ## any purpose.  This copyright notice must be maintained. 
    ## Guido van Rossum And Sundry Contributors are not responsible for 
    ## the consequences of using this software.

    str = char(fread(handle.file, 4, 'char')');
    magic=' ds.';
    invmagic='ds. ';
    magic(1) = char(0);
    invmagic(1) = char(0);
    if strcmp(str, 'dns.') || strcmp(str, magic)
      handle.arch = 'ieee-le';
    elseif strcmp(str, '.snd') || strcmp(str, invmagic)
      handle.arch = 'ieee-be';
    else
      error(msg);
    end
    header = fread(handle.file, 1, 'int32', 0, 'ieee-be');
    len = fread(handle.file, 1, 'int32', 0, 'ieee-be');
    formatid = fread(handle.file, 1, 'int32', 0, 'ieee-be');
    handle.rate = fread(handle.file, 1, 'int32', 0, 'ieee-be');
    handle.channels = fread(handle.file, 1, 'int32', 0, 'ieee-be');
    fseek(handle.file, header-24, SEEK_CUR); % skip file comment

    ## interpret the sample format
    if formatid == 1
      handle.sampleformat = 'ulaw';
      handle.precision = 'uchar';
      handle.bits = 12;
      handle.samples = len;
    elseif formatid == 2
      handle.sampleformat = 'uchar';
      handle.precision = 'uchar';
      handle.bits = 8;
      handle.samples = len;
    elseif formatid == 3
      handle.sampleformat = 'int16';
      handle.precision = 'int16';
      handle.bits = 16;
      handle.samples = len/2;
    elseif formatid == 5
      handle.sampleformat = 'int32';
      handle.precision = 'int32';
      handle.bits = 32;
      handle.samples = len/4;
    elseif formatid == 6
      handle.sampleformat = 'float';
      handle.precision = 'float';
      handle.bits = 32;
      handle.samples = len/4;
    elseif formatid == 7
      handle.sampleformat = 'double';
      handle.precision = 'double';
      handle.bits = 64;
      handle.samples = len/8;
    else
      error(msg);
    end
      
  ## Apple/SGI .aiff format
  elseif strcmp(ext,'aiff') || strcmp(ext,'aif')

    ## Header format obtained from sox/aiff.c
    ## September 25, 1991
    ## Copyright 1991 Guido van Rossum And Sundry Contributors
    ## This source code is freely redistributable and may be used for
    ## any purpose.  This copyright notice must be maintained. 
    ## Guido van Rossum And Sundry Contributors are not responsible for 
    ## the consequences of using this software.
    ##
    ## IEEE 80-bit float I/O taken from
    ##        ftp://ftp.mathworks.com/pub/contrib/signal/osprey.tar
    ##        David K. Mellinger
    ##        dave@mbari.org
    ##        +1-831-775-1805
    ##        fax       -1620
    ##        Monterey Bay Aquarium Research Institute
    ##        7700 Sandholdt Road
 
    ## check the file magic header bytes
    handle.arch = 'ieee-be';
    str = char(fread(handle.file, 4, 'char')');
    if !strcmp(str, 'FORM')
      error(msg);
    end
    len = fread(handle.file, 1, 'int32', 0, handle.arch);
    str = char(fread(handle.file, 4, 'char')');
    if !strcmp(str, 'AIFF')
      error(msg);
    end

    ## skip to the "COMM" section, ignoring everything else
    while (1)
      if feof(handle.file)
      	error(msg);
      end
      str = char(fread(handle.file, 4, 'char')');
      len = fread(handle.file, 1, 'int32', 0, handle.arch);
      if strcmp(str, 'COMM')
	break;
      end
      fseek(handle.file, len, SEEK_CUR);
    end

    ## read the "COMM" section
    handle.channels = fread(handle.file, 1, 'int16', 0, handle.arch);
    frames = fread(handle.file, 1, 'int32', 0, handle.arch);
    handle.bits = fread(handle.file, 1, 'int16', 0, handle.arch);
    exp = fread(handle.file, 1, 'uint16', 0, handle.arch);    % read a 10-byte float
    mant = fread(handle.file, 2, 'uint32', 0, handle.arch);
    mant = mant(1) / 2^31 + mant(2) / 2^63;
    if (exp >= 32768), mant = -mant; exp = exp - 32768; end
    exp = exp - 16383;
    handle.rate = mant * 2^exp;
    fseek(handle.file, len-18, SEEK_CUR);

    ## skip to the "SSND" section, ignoring everything else
    while (1)
      if feof(handle.file)
      	error(msg);
      end
      str = char(fread(handle.file, 4, 'char')');
      len = fread(handle.file, 1, 'int32', 0, handle.arch);
      if strcmp(str, 'SSND')
	break;
      end
      fseek(handle.file, len, SEEK_CUR);
    end
    offset = fread(handle.file, 1, 'int32', 0, handle.arch);
    fread(handle.file, 1, 'int32', 0, handle.arch);
    fseek(handle.file, offset, SEEK_CUR);

    if handle.bits == 8
      handle.precision = 'uchar';
      handle.sampleformat = 'uchar';
      handle.samples = len - 8;
    elseif handle.bits == 16
      handle.precision = 'int16';
      handle.sampleformat = 'int16';
      handle.samples = (len - 8)/2;
    elseif handle.bits == 32
      handle.precision = 'int32';
      handle.sampleformat = 'int32';
      handle.samples = (len - 8)/4;
    else
      error(msg);
    endif
    
  ## file extension unknown
  else
    error('auopen(filename.ext) understands .wav .au and .aiff only');
  end
  handle.dataOffset = ftell(handle.file);
  handle.samples = handle.samples / handle.channels;
endfunction