## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this program; If not, see <http://www.gnu.org/licenses/>.

function [data, handle] = auread(handle, n)
  samplesToRead = n;
  if (handle.sampleOffset+samplesToRead > handle.samples)
      samplesToRead = handle.samples - handle.sampleOffset;
  end
  
  if (handle.precision == 'int24')
    data = fread(handle.file, 3*samplesToRead*handle.channels, 'uint8', 0, handle.arch);
    if (handle.arch == 'ieee-le')
      data = data(1:3:end) + data(2:3:end) * 2^8 + cast(typecast(cast(data(3:3:end), 'uint8'), 'int8'), 'double') * 2^16;
    else
      data = data(3:3:end) + data(2:3:end) * 2^8 + cast(typecast(cast(data(1:3:end), 'uint8'), 'int8'), 'double') * 2^16;
    endif
  else
    data = fread(handle.file, samplesToRead*handle.channels, handle.precision, 0, handle.arch);
  endif

  ## convert samples into range [-1, 1)
  if strcmp(handle.sampleformat, 'alaw')
    alaw = [ \
	     -5504,  -5248,  -6016,  -5760,  -4480,  -4224,  -4992,  -4736, \
	     -7552,  -7296,  -8064,  -7808,  -6528,  -6272,  -7040,  -6784, \
	     -2752,  -2624,  -3008,  -2880,  -2240,  -2112,  -2496,  -2368, \
	     -3776,  -3648,  -4032,  -3904,  -3264,  -3136,  -3520,  -3392, \
	    -22016, -20992, -24064, -23040, -17920, -16896, -19968, -18944, \
	    -30208, -29184, -32256, -31232, -26112, -25088, -28160, -27136, \
	    -11008, -10496, -12032, -11520,  -8960,  -8448,  -9984,  -9472, \
	    -15104, -14592, -16128, -15616, -13056, -12544, -14080, -13568, \
	      -344,   -328,   -376,   -360,   -280,   -264,   -312,   -296, \
	      -472,   -456,   -504,   -488,   -408,   -392,   -440,   -424, \
	       -88,    -72,   -120,   -104,    -24,     -8,    -56,    -40, \
	      -216,   -200,   -248,   -232,   -152,   -136,   -184,   -168, \
	     -1376,  -1312,  -1504,  -1440,  -1120,  -1056,  -1248,  -1184, \
	     -1888,  -1824,  -2016,  -1952,  -1632,  -1568,  -1760,  -1696, \
	      -688,   -656,   -752,   -720,   -560,   -528,   -624,   -592, \
	      -944,   -912,  -1008,   -976,   -816,   -784,   -880,   -848 ];
    alaw = ([ alaw,-alaw]+0.5)/32767.5;
    data = alaw(data+1);
  elseif strcmp(handle.sampleformat, 'ulaw')
    data = mu2lin(data, 0);
  elseif strcmp(handle.sampleformat, 'uchar')
    data = data * 2^(-7);
  elseif strcmp(handle.sampleformat, 'int16')
    data = data * 2^(-15);
  elseif strcmp(handle.sampleformat, 'int24')
    data = data * 2^(-23);
  elseif strcmp(handle.sampleformat, 'int32')
    data = data * 2^(-31);
  end
  data = reshape(data, handle.channels, length(data)/handle.channels)';
  handle.sampleOffset += length(data)/handle.channels;
endfunction